/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
var DependencyGraphAnalyzer = {};

DictionaryEntry = function(k, v) {
    this.key=k;
    this.value=v;
    this.valueOf=function(){
        return this.value;
    };
    this.toString=function(){
        return String(this.value);
    };
}

Dictionary = function(){
    var items={};
    this.count=0;

    this.add=function(k, v){
        var b=(k in items);
        items[k]=new DictionaryEntry(k,v);
        if(!b){
            this.count++;
        }
    };
    this.item=function(k){
        if(k in items){
            return items[k].valueOf();
        }
        return undefined;
    };
}

scanForCircularDependencies = function(indexGraph, jsModule, seen) {
    seen.push(jsModule.i);
    for (var i = 0; i < jsModule.d.length; i++) {
        var childJsModule = indexGraph.item(jsModule.d[i]);

        if (childJsModule.scanned == false) {
            if (contains(childJsModule.i, seen) == true) {
                var msg = "";
                for (var j = 0; j < seen.length; j++) {
                    var jsm = indexGraph.item(seen[j]);
                    msg += jsm.m;
                    msg += "->";
                }
                msg += childJsModule.m;

                print("A Circular Dependency has been found ["+childJsModule.m+"] ["+msg+']');
            }
            else {
                scanForCircularDependencies(indexGraph, childJsModule, seen);
            }
        }
    }
    jsModule.scanned = true;
    seen.pop();
}

contains = function(item, stack) {
    for (var i = 0; i < stack.length; i++) {
        if (stack[i] == item) {
           return true;
        }
    }
    return false;
}

DependencyGraphAnalyzer.analyze = function(filename) {
    var reader = new Packages.java.io.BufferedReader(new Packages.java.io.FileReader(filename));
    var sb = new java.lang.StringBuffer();
    var line = "";
    while((line = reader.readLine()) !== null){
        sb.append(line);
        //sb.append(java.lang.System.getProperty("line.separator"));
    }
    try {
        var data = eval("(" + new String(sb.toString())+ ")");

        var indexGraph = new Dictionary();
        var uriGraph = new Dictionary();
        var moduleGraph = new Dictionary();

        for (var i = 0; i < data.j.length; i++) {
            var jsModule = data.j[i];
            jsModule.scanned = false;
            indexGraph.add(jsModule.i, jsModule);
            uriGraph.add(jsModule.u, jsModule);
            moduleGraph.add(jsModule.m, jsModule);
            /*
            print("jsModule index = ["+jsModule.i);
            print("jsModule uri = ["+jsModule.u);
            print("jsModule module name = ["+jsModule.m);
            print("jsModule loaded = ["+jsModule.l);

            for (var j = 0; j < jsModule.d.length; j++) {
                print("jsModule dependency = ["+jsModule.d[j]+"]");
            }
            */
        }

        for (var i = 0; i < data.j.length; i++) {
            var seen = [];
            scanForCircularDependencies(indexGraph, data.j[i], seen);
        }
    }
    catch (exception) {
        print("exception: " +exception);
    }
}

