/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.internal.SimpleToolbar");

dojo.require("dijit.Toolbar");

(function() {

/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 */
dojo.declare("jazz.ui.internal.SimpleToolbar", [dijit._Widget, dijit._Templated], {
	
	templateString: "<div class='jazz-ui-SimpleToolbar'></div>",
	
	postCreate: function() {
		this.items = new Array();
		this._events = new Array();
	},
	
	// Adds a basic toolbar button with a 16x16 icon and hover state.
	addButton: function(context, callback, args) {
		//args: label, imgSrc, index
		if (!args.imgSrc) {
			console.error("Toolbar: No image URL passed. Skipping.");
			return;
		}
		var button = document.createElement("a");
		button.className = "jazz-ui-SimpleToolbar-button"
		button.href = "#";
		button.title = args.label;
		
		var img = document.createElement("img");
		img.src = args.imgSrc;
		img.alt = args.label;
		button.appendChild(img);
		
		if (callback) {
			this._events.push(this.connect(button, "onclick", function(event) {
				event.preventDefault();
				dojo.hitch(context, callback)(event);
			}));
		}
		try {
			return this._insert({domNode: button}, args.index);
		} finally {
			button = null;
			img = null;
		}
	},
	addSeparator: function(index) {
		var separator = document.createElement("div");
		separator.className = "jazz-ui-SimpleToolbar-separator";
		return this._insert({domNode: separator}, index);
	},
	addWidget: function(widget, args) {
		// widget:  must have domNode
		// args:  isButton, index
		if (!args) args = {};
		var domNode = widget.domNode;
		if (args.isButton === true) {
			dojo.addClass(widget.domNode,"jazz-ui-SimpleToolbar-button");
		}
		return this._insert(widget, args.index);
	},
	getItem: function(index) {
		return this.items[index] | null;
	},
	removeItem: function(widget) {
		var item;
		for (var i = 0; i < this.items.length; i++) {
			if (this.items[i] === widget) {
				item = this.items[i];
				if (item && item.domNode) {
					item.domNode.parentNode.parentNode.removeChild(item.domNode.parentNode);
					return this.items.splice(i,1);
				}
			}
		}
		return null;
	},
	hideItem: function(widget) {
		if (widget && widget.domNode && widget.domNode.parentNode) {
			widget.domNode.parentNode.style.display = "none";	
		}
	},
	showItem: function(widget) {
		if (widget && widget.domNode && widget.domNode.parentNode) {
			widget.domNode.parentNode.style.display = "";	
		}		
	},
	updateButton: function(widget, args) {
		// args: label, imgSrc
		if (widget && widget.domNode && widget.domNode.firstChild && widget.domNode.firstChild.nodeName.toUpperCase() == "IMG") {
			var img = widget.domNode.firstChild;
			if (args.label || args.label === "") {
				img.alt = args.label;
				widget.domNode.title = args.label;
			}
			if (args.imgSrc)
				img.src = args.imgSrc;
		}
	},
	_insert: function(widget, index) {
		if (!widget) return null;
		
		var div = document.createElement("div");
		dojo.addClass(div, "jazz-ui-SimpleToolbar-item");
		div.appendChild(widget.domNode);
		
		if (index != null && index < this.items.length) {
			this.domNode.insertBefore(div,this.items[index].domNode.parentNode);
			this.items.splice(index,0,widget);
			return widget;
		} else {
			this.domNode.appendChild(div);
			this.items.push(widget);
			return widget;
		}
	}
});
})();