/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.UserProfilePage");

/*
 * Imports 
 */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.UserEditor");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UserProfilePageMessages");


(function() {

/*
 * Aliases 
 */
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var UserEditor = com.ibm.team.repository.web.ui.internal.admin.users.UserEditor;

var ActionRegistry = null; 

/*
 * Class 
 */
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.UserProfilePage",
	[dijit._Widget, dijit._Templated], {
	
	// Properties //----------------------------------------------------------->
	
	contentWidget: null,
	pictures: null,
	
	/*
	 * Templates
	 */		
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/UserProfilePage.html"),


	// Public Methods //------------------------------------------------------->
	
	/*
 	 * @param Object args
 	 * 
 	 * @return void
 	 */	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "UserProfilePageMessages");
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	postCreate: function() {
		this.inherited(arguments);

		// Perform a late binding to the action registry to ensure that the 
		// work bench is instantiated  
		ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		ActionRegistry.registerAction("com.ibm.team.repository.viewUserProfile", this, "viewUserProfile");
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		if (this.pictures) {
			delete this.pictures;
		}
		
		this._destroyContentWidget();
		
		this.inherited(arguments);
	},		
	
	/*
 	 * @param Object args
 	 * 
 	 * @return void
 	 */		
	viewUserProfile: function(args) {
		// Update title
		this._updateBrowserTitle();

		// Validate item UUID		
		if (!args.itemId) {
			dojo.publish(net.jazz.ajax.internal.events.viewErrorPage, 
				[
					this.messages.msgErrorOccurred, 
					this.messages.msgErrorMissingParameter, 
					false
				]); 
			return;
		}
		
		// Create page
		this._createContentWidget(args.itemId);
		
		// Preload images
		setTimeout(dojo.hitch(this, this._preloadPrimaryImages), 0);
	},

	// Private Methods //------------------------------------------------------>
	
	/*
 	 * @param String itemId
 	 * 
 	 * @return void
 	 */		
	_createContentWidget: function(itemId) {

		this._destroyContentWidget();
		
		this.contentWidget = new UserEditor({
			itemId: itemId
		});
		this.domNode.appendChild(this.contentWidget.domNode);		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_destroyContentWidget: function() {
		if (this.contentWidget) {
			this.contentWidget.destroy();
			delete this.contentWidget;
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_updateBrowserTitle: function() {
		PlatformUI.getWorkbench().setTitle(this.messages.msgUserProfile);
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_preloadPrimaryImages: function() {
		
		if (this.pictures) {
			return;
		} 
		this.pictures = new Array();
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/userIcon.png"));
	}

});

})();
