/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.WizardForm");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.validate.regexp");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardFormMessages");


(function() {

// The WizardForm is the common superclass for forms to be used in WizardSteps. 
// The key value add of this class is to provide a common data validation routine 
// that introspects the form's data and metadata to determine if it's ready to be submitted.

// WizardForm calls down to these subsclasses' runAction method when the user presses the
// form's action button (e.g. "Create User", "Test connection").


var VALIDATORS = {
	email: {
		regex: new RegExp(dojox.regexp.emailAddress()),
		messageKey: "invalidEmail"
	}
};

var errorMessages;
var messages;
dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.WizardForm",  [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/WizardForm.html"),

	postCreate: function(){
		errorMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardFormMessages");
		messages = this.messages;
		if(isNode(this.description)) {
			this.descriptionArea.appendChild(this.description);
		} else {
			this.descriptionArea.innerHTML = this.description;
		}
		this.descriptionArea.className = "description-area";
		
		if (!this.tableContent) {
			dojo.toggleClass(this.OuterPropertyTable, "noDisplay", true);
			return;
		}
		this.connect(this.propertyTableBody,"onkeyup","handleKeyUp");
		
		var headerObjs = this.tableContent.header;
		for(var i=0; i<headerObjs.length; i++) {
			var hObj = headerObjs[i];
			var th = document.createElement("th");
			if(hObj.type === "data"){
				th.className = "data";
			} 
			if(hObj.className){
				th.className = hObj.className;
			}
			dojo.addClass(th, "column-"+i);
			this.propertyTableHeaderRow.appendChild(th);
			var labelString = hObj.label;
			if (!labelString) {
				labelString = this.messages[hObj.labelKey];
			}
			th.appendChild(document.createTextNode(labelString));
		}
		
		var tableDefRows = this.tableContent.rows;
		this.dataFields = [];
		var inputMap = {};
		var mustMatchMap = {};
		for(var i=0; i<tableDefRows.length; i++){
			var row = document.createElement("tr");
			var tableDefRow = tableDefRows[i];
			this.propertyTableBody.appendChild(row);
			for (var j=0; j<headerObjs.length; j++) {
				var colType = headerObjs[j].type;
				var td = row.insertCell(-1);
				if(colType === "data"){
					td.className = "data";
					var input = createInputElement(tableDefRow);
					this.dataFields.push(input);
					td.appendChild(input);
					this.connect(input,"onblur","handleChange");
					// store a back-pointer to containing row
					input._jazzRow = row;
					// if this field must match another, store a reference in
					// the 'must match' map for later
					var name = tableDefRow.name;
					inputMap[name] = input;
					if(tableDefRow.mustMatch){
						mustMatchMap[name] = tableDefRow.mustMatch;
					}
					// if this field has a particular data type, store a
					// reference to the relevant validator
					if(tableDefRow.dataType){
						input._jazzValidator = VALIDATORS[tableDefRow.dataType];
					}
					//if row comes with an example, add it
					if(tableDefRow.exampleKey) {
						var div = document.createElement("div");
						div.className = "propertyExample";
						var node = document.createTextNode(this.messages[tableDefRow.exampleKey]);
						div.appendChild(node);
						td.appendChild(div);	
					} 
				} else {
					var val = tableDefRow[colType];
					if (!val) {
						valKey = tableDefRow[colType+"Key"];
						val = this.messages[valKey];
					}
					if (colType === "description" ) {
						td.className = "inline-description";
					} 
					if(isNode(val)) {
						var node = val;
					} else {
						node = document.createTextNode(val);
					}
					td.appendChild(node);
				}
			}
		}
		// for each member of the mustMatchMap, create a reference in the source input
		// to the input the source input must match
		for (var inputName in mustMatchMap) {
			var srcInput = inputMap[inputName];
			var matchEntry = mustMatchMap[inputName];
			matchEntry.matchField = inputMap[matchEntry.matchName];
			
			srcInput._jazzMustMatch = matchEntry;
		}
	},
	
	handleKeyUp: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this.validate);
			var timeout = 2000;
			if (e.target._jazzRow  && e.target._jazzRow.hasError) {
				timeout = 500;
			}
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	handleChange: function(e) {
		this.validate();
	},
	
	validate: function() {
		var formStatus = {
			isComplete: true,
			isValid: true,
			type: "INFO"
		};
		if (this.dataFields) {
			for (var i = 0; i < this.dataFields.length; i++) {
				var input = this.dataFields[i];
				var fieldStatus = checkField(input);
				if (fieldStatus.isEmpty) {
					formStatus.isComplete = false;
				}
				var td = input.parentNode;
				var msgDiv = td.childNodes.length > 1 ? td.childNodes[1] : null;
				if (fieldStatus.isValid) {
					dojo.removeClass(input._jazzRow, "hasError");
					input._jazzRow.hasError = false;
					
					//Remove message divs, but make sure they are ACTUALLY msgDivs. 
					//This will prevent the propertyExample divs from getting accidentally removed.
					if (msgDiv && msgDiv.className != "propertyExample") {
						td.removeChild(msgDiv);
					}
				}
				else {
					formStatus.isValid = false;
					formStatus.type = "ERROR";
					dojo.addClass(input._jazzRow, "hasError");
					input._jazzRow.hasError = true;
					if (!msgDiv) {
						msgDiv = document.createElement("div");
					}
					else {
						msgDiv.innerHTML = "";
					}
					msgDiv.appendChild(document.createTextNode(fieldStatus.error));
					td.appendChild(msgDiv);
				}
			}
		}
		clearTimeout(this.onKeyUpTimout);
		formStatus.overrideNext = formStatus.isComplete && formStatus.isValid;
		return formStatus;
	},
	
	getData: function() {
		var obj = {};
		if (!this.dataFields) {
			return obj;
		}
		for(var i=0; i<this.dataFields.length; i++){
			var name = this.dataFields[i].name;
			if (this.dataFields[i].type == "checkbox") {
				obj[name] = this.dataFields[i].checked;
			} else {
				obj[name] = dojo.trim(this.dataFields[i].value);
			}
		}
		return obj;
	}, 
	
	clearData: function() {
		if (!this.dataFields) {
			return;
		}
		for(var i=0; i<this.dataFields.length; i++){
			if (this.dataFields[i].type == "checkbox") {
				this.dataFields[i].checked = false;
			} else {
				this.dataFields[i].value = "";
			}
		}
	}, 
	
	disableFields: function (value) {
		if (!this.dataFields) {
			return;
		}
		for(var i=0; i<this.dataFields.length; i++){
			this.dataFields[i].disabled = value;
		}
	}
});

function checkField(input) {
	var isValid = true;
	var isEmpty = input.value === "";
	var error = null;
	
	// check if the field must match another field and if so check it
	if(isValid && !isEmpty && input._jazzMustMatch){
		var mustMatchValue = input._jazzMustMatch.matchField.value;
		if(input.value !== dojo.trim(mustMatchValue)){
			isValid = false;
			error = input._jazzMustMatch.errorMessage;
			if (!error) {
				error = messages[input._jazzMustMatch.errorMessageKey];
			}
			if (!error) {
				error = errorMessages.mismatchError
			}
		}
	}
	
	// check to see if the field has any validator and if so if it's valid
	if(isValid && !isEmpty && input._jazzValidator){
		var match = input._jazzValidator.regex.exec(input.value);
		if(!match){
			isValid = false;
			error = input._jazzValidator.message;
			if (!error) {
				error = errorMessages[input._jazzValidator.messageKey];
			}
		}
	}
	
	return {
		isValid: isValid,
		isEmpty: isEmpty,
		error: error
	};
}

function isNode(o){
	return o && o.nodeType && o.cloneNode;
}

function createInputElement(args){
	var input = document.createElement("input");
	// set the type of the field if necesary
	if (args.type) {
		input.type = args.type;
	}
	// disable the field if necessary
	if (args.disabled) {
		input.disabled = true;
	}
	input.name = args.name;
	if (args.type === "checkbox") {
		input.id = args.name;
		input.className = "checkInput";
	}
	return input;
}

com.ibm.team.repository.web.ui.internal.wizard.WizardForm.createInputElement = createInputElement;
})();